// ===========================================================================
// CONTENT  : TEST CLASS IdTokenConverterTest
// AUTHOR   : Manfred Duchrow
// VERSION  : 1.0 - 28/08/2014
// HISTORY  :
//  28/08/2014  mdu  CREATED
//
// Copyright (c) 2014, by MDCS. All rights reserved.
// ===========================================================================
package org.pfsw.oidc.token;

// ===========================================================================
// IMPORTS
// ===========================================================================
import static org.junit.Assert.*;

import org.junit.Test;
import org.pfsw.bif.text.ICharsets;
import org.pfsw.junit.ExceptionAssertion;
import org.pfsw.jwt.JwtConversionException;
import org.pfsw.text.TimeValue;
import org.pfsw.text.json.JsonObject;
import org.pfsw.text.json.JsonParseException;
import org.pfsw.text.json.JsonUtil;
import org.pfsw.text.json.SimpleJsonParser;
import org.pfsw.util.Base64Converter;

public class IdTokenConverterTest
{
  @Test
  public void test_convert_sample_1() throws JwtConversionException
  {
    IdTokenConverter converter = new IdTokenConverter();
    IdToken idToken;
    
    idToken = converter.convert(this.createSampleIDTokenBase64String1());
    assertEquals("https://openid.autserver.com:9443/oauth2endpoints/token", idToken.getIssuer());
    assertEquals("halvar@unittest.net", idToken.getSubject());
    assertEquals(0, idToken.getAuthenticationMethods().length);
    assertEquals(1, idToken.getAudiences().length);
    assertEquals("aLHmcuWwtgDg3fvRAMp8Sj0sqUIa", idToken.getAudiences()[0]);
    assertEquals("aLHmcuWwtgDg3fvRAMp8Sj0sqUIa", idToken.getAuthorizedParty());
    assertEquals(new TimeValue(475807619000L), idToken.getIssuedTime());
    assertTrue(idToken.isExpired());
  } 

  @Test
  public void test_convertToInternal_sample_2() throws JwtConversionException
  {
    IdTokenConverter converter = new IdTokenConverter();
    IdToken jwt;
    
    jwt = converter.convert(this.createSampleIDTokenBase64String2());
    assertEquals("https://openid.autserver.com:9443/oauth2endpoints/token", jwt.getIssuer());
    assertEquals("snorre@unittest.net", jwt.getSubject());
    assertEquals(3, jwt.getAuthenticationMethods().length);
    assertEquals("uno", jwt.getAuthenticationMethods()[0]);
    assertEquals("dos", jwt.getAuthenticationMethods()[1]);
    assertEquals("tres", jwt.getAuthenticationMethods()[2]);
    assertEquals(2, jwt.getAudiences().length);
    assertEquals("aLHmcuWwtgDg3fvRAMp8Sj0sqUIa", jwt.getAudiences()[0]);
    assertEquals("r6nHgd8Op9baz69BAi", jwt.getAudiences()[1]);
    assertEquals("aLHmcuWwtgDg3fvRAMp8Sj0sqUIa", jwt.getAuthorizedParty());
    assertEquals(new TimeValue(475801111000L), jwt.getIssuedTime());
    assertTrue(jwt.isExpired());
  }

  @Test
  public void test_convertToInternal_missing_filed()
  {
    final IdTokenConverter converter = new IdTokenConverter();
    final String jsonString;
    
    jsonString = this.createSampleIDTokenBase64String3();

    ExceptionAssertion assertion = new ExceptionAssertion()
    {
      @Override
      public void run() throws Exception
      {
        converter.convert(jsonString);        
      }
    };
    assertion.assertException(JwtConversionException.class);
  }  
  
  // =========================================================================
  // PROTECTED INSTANCE METHODS
  // =========================================================================
  protected String createSampleIDTokenBase64String1() 
  {
    return  Base64Converter.encodeBase64url(this.getJoseHeaderSample1(), ICharsets.UTF_8) + "."
        + Base64Converter.encodeBase64url(this.getSampleIDTokenJSONText1(), ICharsets.UTF_8);
  } 
  
  protected String createSampleIDTokenBase64String2() 
  {
    return Base64Converter.encodeBase64url(this.getJoseHeaderSample1(), ICharsets.UTF_8) + "."
        + Base64Converter.encodeBase64url(this.getSampleIDTokenJSONText2(), ICharsets.UTF_8);
  } 
  
  protected String createSampleIDTokenBase64String3() 
  {
    JsonObject jsonObject;
    
    jsonObject = this.createSampleIDTokenJsonObject(this.getSampleIDTokenJSONText1());
    jsonObject.remove("aud");
    return Base64Converter.encodeBase64url(JsonUtil.current().mapToJSON(jsonObject), ICharsets.UTF_8);
  } 
  
  protected JsonObject createSampleIDTokenJsonObject(String jsonText) 
  {
    SimpleJsonParser parser = new SimpleJsonParser();
    
    try
    {
      return parser.parseObject(jsonText);
    }
    catch (JsonParseException ex)
    {
      ex.printStackTrace();
      fail();
      return null;
    }
  } 
  
  protected String getJoseHeaderSample1()
  {
    return "{ \"alg\" : \"RS256\", \"typ\" : \"JWT\" }";
  }
  
  protected String getSampleIDTokenJSONText1() 
  {
    return "{\"exp\":479407619,\"azp\":\"aLHmcuWwtgDg3fvRAMp8Sj0sqUIa\",\"sub\":\"halvar@unittest.net\",\"aud\":\"aLHmcuWwtgDg3fvRAMp8Sj0sqUIa\",\"iss\":\"https://openid.autserver.com:9443/oauth2endpoints/token\",\"iat\":475807619}";
  } 
  
  protected String getSampleIDTokenJSONText2() 
  {
    return "{\"exp\":479407781,\"azp\":\"aLHmcuWwtgDg3fvRAMp8Sj0sqUIa\",\"amr\":[\"uno\",\"dos\",\"tres\"],\"sub\":\"snorre@unittest.net\",\"aud\":[\"aLHmcuWwtgDg3fvRAMp8Sj0sqUIa\",\"r6nHgd8Op9baz69BAi\"],\"iss\":\"https://openid.autserver.com:9443/oauth2endpoints/token\",\"iat\":475801111}";
  } 
  
} 
